using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Data.SqlClient;

namespace Akadia.CustomerTransactions
{
	public class CustTrans : System.Windows.Forms.Form
	{
		// Visual Controls
		private System.ComponentModel.Container components = null;
		private TextBox txtPhone;
		private TextBox txtContactTitle;
		private TextBox txtZip;
		private TextBox txtCity;
		private TextBox txtAddress;
		private TextBox txtContactName;
		private TextBox txtCompanyName;
		private TextBox txtCompanyID;
		private Label lblCompanyID;
		private Button btnNew;
		private TextBox txtCustomerName;
		private Button btnUpdate;
		private Button btnDelete;
		private Label lblCity;
		private Label lblZip;
		private Label lblContactTitle;
		private Label lblPhone;
		private Label lblAddress;
		private Label lblCompanyName;
		private Label lblContactName;
		private Label lblNewCustomerName;
		private ListBox listBoxCustomers;
		private GroupBox grpBoxUpdateCustomer;
		private GroupBox grpBoxNewCustomer;
		private Label lblMessage;

		// The DataSet, DataAdapter, and DataTable are members
		// so that we can access them from any member method.
		private SqlDataAdapter _dataAdapter;
		private DataSet _dataSet;
		private DataTable _dataTable;

		// **** Constructor
		public CustTrans()
		{
			// Create the visual Controls
			InitializeComponent();

			// Setup DB Connection **** ADJUST IT FOR YOUR USE ****
			string connectionString = "server=xeon;" + 
				"uid=sa; pwd=manager; database=northwind";
			string commandString = "Select * from Customers";
			_dataAdapter =  new SqlDataAdapter(commandString, connectionString);

			// Create INSERT, UPDATE, DELETE "Template" Commands
			InitializeCommands();

			// Populate DataSet
			_dataSet = new DataSet();
			_dataAdapter.Fill(_dataSet,"Customers");

			// Fill the list box with columns from the Dataset
			this.PopulateListBox();
		}

		// *** Bind parameters to appropriate columns.
		// Note: The params keyword lets you specify a method
		// parameter that takes an argument where the number
		// of arguments is variable.
		private void AddParms(SqlCommand cmd, params string[] cols) 
		{
			// Add each parameter. Note that each colum in
			// table "Customers" is of type VARCHAR !
			foreach (String column in cols) 
			{
				cmd.Parameters.Add(
					"@" + column, SqlDbType.Char, 0, column);
			}
		}

		// *** Setup DataAdapter with expicit INSERT, UPDATE, 
		// DELETE "Template" Commands.
		private void InitializeCommands()
		{
			// Reuse the SelectCommand's Connection.
			SqlConnection con =
				(SqlConnection) _dataAdapter.SelectCommand.Connection;

			// Create an explicit, reusable INSERT command
			_dataAdapter.InsertCommand = con.CreateCommand();
			_dataAdapter.InsertCommand.CommandText =
				"INSERT INTO customers " +
				"(CustomerId, CompanyName, ContactName, ContactTitle, " +
				"Address, City, PostalCode, Phone) " +
				"VALUES(@CustomerId, @CompanyName, @ContactName, " +
				"@ContactTitle, @Address, @City, @PostalCode, @Phone)";

			// Bind parameters to appropriate columns for INSERT command
			this.AddParms(_dataAdapter.InsertCommand,
				"CustomerId", "CompanyName", "ContactName", "ContactTitle",
				"Address", "City", "PostalCode", "Phone");

			// Create an explicit UPDATE command
			_dataAdapter.UpdateCommand = con.CreateCommand();
			_dataAdapter.UpdateCommand.CommandText = 
				"UPDATE customers " +
				"SET CompanyName = @CompanyName " +
				"WHERE CustomerID = @CustomerId";

			// Bind parameters to appropriate columns for UPDATE command
			this.AddParms(_dataAdapter.UpdateCommand, "CompanyName", "CustomerID");

			// Create an explicit DELETE command
			_dataAdapter.DeleteCommand = con.CreateCommand();
			_dataAdapter.DeleteCommand.CommandText =
				"DELETE FROM customers " +
				"WHERE customerID = @CustomerId";

			// Bind parameters to appropriate columns for DELETE command
			AddParms(_dataAdapter.DeleteCommand, "CustomerID");
		}

		// *** Fill the list box with columns from the Dataset
		private void PopulateListBox()
		{
			// Get the only one Table from the Dataset
			_dataTable = _dataSet.Tables[0];
			listBoxCustomers.Items.Clear();

			// Loop through the Dataset and add each Row
			// to the ListBox
			foreach (DataRow dataRow in _dataTable.Rows)
			{
				listBoxCustomers.Items.Add(
					dataRow["CustomerId"] + ", " +
					dataRow["CompanyName"] + ", " +
					dataRow["ContactName"] + ", " +
					dataRow["ContactTitle"] + ", " +
					dataRow["Address"] + ", " +
					dataRow["City"] + ", " +
					dataRow["PostalCode"] + ", " +
					dataRow["Phone"] + ")" );
			}
		}

		// **** Set all the text fields to empty strings
		private void ClearFields()
		{
			txtCompanyID.Text = "";
			txtCompanyName.Text = "";
			txtContactName.Text = "";
			txtContactTitle.Text = "";
			txtAddress.Text = "";
			txtCity.Text = "";
			txtZip.Text = "";
			txtPhone.Text = "";
		}

		// **** Create the visual Controls, let the Designer do
		// this and DO NOT CHANGE MANUALLY.
		private void InitializeComponent()
		{
			this.txtCustomerName = new System.Windows.Forms.TextBox();
			this.txtCity = new System.Windows.Forms.TextBox();
			this.txtCompanyID = new System.Windows.Forms.TextBox();
			this.btnUpdate = new System.Windows.Forms.Button();
			this.txtContactName = new System.Windows.Forms.TextBox();
			this.txtZip = new System.Windows.Forms.TextBox();
			this.btnDelete = new System.Windows.Forms.Button();
			this.txtContactTitle = new System.Windows.Forms.TextBox();
			this.txtAddress = new System.Windows.Forms.TextBox();
			this.txtCompanyName = new System.Windows.Forms.TextBox();
			this.lblCity = new System.Windows.Forms.Label();
			this.lblZip = new System.Windows.Forms.Label();
			this.lblContactTitle = new System.Windows.Forms.Label();
			this.lblPhone = new System.Windows.Forms.Label();
			this.lblNewCustomerName = new System.Windows.Forms.Label();
			this.lblAddress = new System.Windows.Forms.Label();
			this.listBoxCustomers = new System.Windows.Forms.ListBox();
			this.txtPhone = new System.Windows.Forms.TextBox();
			this.btnNew = new System.Windows.Forms.Button();
			this.lblCompanyID = new System.Windows.Forms.Label();
			this.lblCompanyName = new System.Windows.Forms.Label();
			this.lblContactName = new System.Windows.Forms.Label();
			this.grpBoxUpdateCustomer = new System.Windows.Forms.GroupBox();
			this.grpBoxNewCustomer = new System.Windows.Forms.GroupBox();
			this.lblMessage = new System.Windows.Forms.Label();
			this.grpBoxUpdateCustomer.SuspendLayout();
			this.SuspendLayout();
			// 
			// txtCustomerName
			// 
			this.txtCustomerName.Location = new System.Drawing.Point(26, 170);
			this.txtCustomerName.Name = "txtCustomerName";
			this.txtCustomerName.Size = new System.Drawing.Size(275, 20);
			this.txtCustomerName.TabIndex = 4;
			this.txtCustomerName.Text = "";
			// 
			// txtCity
			// 
			this.txtCity.Location = new System.Drawing.Point(326, 266);
			this.txtCity.Name = "txtCity";
			this.txtCity.Size = new System.Drawing.Size(160, 20);
			this.txtCity.TabIndex = 15;
			this.txtCity.Text = "";
			// 
			// txtCompanyID
			// 
			this.txtCompanyID.Location = new System.Drawing.Point(111, 237);
			this.txtCompanyID.Name = "txtCompanyID";
			this.txtCompanyID.Size = new System.Drawing.Size(160, 20);
			this.txtCompanyID.TabIndex = 7;
			this.txtCompanyID.Text = "";
			// 
			// btnUpdate
			// 
			this.btnUpdate.Location = new System.Drawing.Point(308, 169);
			this.btnUpdate.Name = "btnUpdate";
			this.btnUpdate.Size = new System.Drawing.Size(179, 23);
			this.btnUpdate.TabIndex = 0;
			this.btnUpdate.Text = "Update selected Customer Name";
			this.btnUpdate.Click += new System.EventHandler(this.btnUpdate_Click);
			// 
			// txtContactName
			// 
			this.txtContactName.Location = new System.Drawing.Point(111, 295);
			this.txtContactName.Name = "txtContactName";
			this.txtContactName.Size = new System.Drawing.Size(160, 20);
			this.txtContactName.TabIndex = 11;
			this.txtContactName.Text = "";
			// 
			// txtZip
			// 
			this.txtZip.Location = new System.Drawing.Point(326, 295);
			this.txtZip.Name = "txtZip";
			this.txtZip.Size = new System.Drawing.Size(160, 20);
			this.txtZip.TabIndex = 17;
			this.txtZip.Text = "";
			// 
			// btnDelete
			// 
			this.btnDelete.Location = new System.Drawing.Point(494, 76);
			this.btnDelete.Name = "btnDelete";
			this.btnDelete.Size = new System.Drawing.Size(54, 23);
			this.btnDelete.TabIndex = 2;
			this.btnDelete.Text = "Delete";
			this.btnDelete.Click += new System.EventHandler(this.btnDelete_Click);
			// 
			// txtContactTitle
			// 
			this.txtContactTitle.Location = new System.Drawing.Point(111, 324);
			this.txtContactTitle.Name = "txtContactTitle";
			this.txtContactTitle.Size = new System.Drawing.Size(160, 20);
			this.txtContactTitle.TabIndex = 12;
			this.txtContactTitle.Text = "";
			// 
			// txtAddress
			// 
			this.txtAddress.Location = new System.Drawing.Point(326, 237);
			this.txtAddress.Name = "txtAddress";
			this.txtAddress.Size = new System.Drawing.Size(160, 20);
			this.txtAddress.TabIndex = 13;
			this.txtAddress.Text = "";
			// 
			// txtCompanyName
			// 
			this.txtCompanyName.Location = new System.Drawing.Point(111, 266);
			this.txtCompanyName.Name = "txtCompanyName";
			this.txtCompanyName.Size = new System.Drawing.Size(160, 20);
			this.txtCompanyName.TabIndex = 9;
			this.txtCompanyName.Text = "";
			// 
			// lblCity
			// 
			this.lblCity.Location = new System.Drawing.Point(271, 270);
			this.lblCity.Name = "lblCity";
			this.lblCity.RightToLeft = System.Windows.Forms.RightToLeft.Yes;
			this.lblCity.Size = new System.Drawing.Size(49, 13);
			this.lblCity.TabIndex = 14;
			this.lblCity.Text = "City";
			// 
			// lblZip
			// 
			this.lblZip.Location = new System.Drawing.Point(271, 299);
			this.lblZip.Name = "lblZip";
			this.lblZip.RightToLeft = System.Windows.Forms.RightToLeft.Yes;
			this.lblZip.Size = new System.Drawing.Size(49, 13);
			this.lblZip.TabIndex = 16;
			this.lblZip.Text = "Zip";
			// 
			// lblContactTitle
			// 
			this.lblContactTitle.Location = new System.Drawing.Point(24, 327);
			this.lblContactTitle.Name = "lblContactTitle";
			this.lblContactTitle.RightToLeft = System.Windows.Forms.RightToLeft.Yes;
			this.lblContactTitle.Size = new System.Drawing.Size(82, 13);
			this.lblContactTitle.TabIndex = 28;
			this.lblContactTitle.Text = "Contact Title";
			// 
			// lblPhone
			// 
			this.lblPhone.Location = new System.Drawing.Point(278, 327);
			this.lblPhone.Name = "lblPhone";
			this.lblPhone.RightToLeft = System.Windows.Forms.RightToLeft.Yes;
			this.lblPhone.Size = new System.Drawing.Size(42, 13);
			this.lblPhone.TabIndex = 20;
			this.lblPhone.Text = "Phone";
			// 
			// lblNewCustomerName
			// 
			this.lblNewCustomerName.Location = new System.Drawing.Point(27, 154);
			this.lblNewCustomerName.Name = "lblNewCustomerName";
			this.lblNewCustomerName.RightToLeft = System.Windows.Forms.RightToLeft.No;
			this.lblNewCustomerName.Size = new System.Drawing.Size(147, 14);
			this.lblNewCustomerName.TabIndex = 22;
			this.lblNewCustomerName.Text = "Update Customer Name";
			// 
			// lblAddress
			// 
			this.lblAddress.Location = new System.Drawing.Point(271, 241);
			this.lblAddress.Name = "lblAddress";
			this.lblAddress.RightToLeft = System.Windows.Forms.RightToLeft.Yes;
			this.lblAddress.Size = new System.Drawing.Size(49, 13);
			this.lblAddress.TabIndex = 26;
			this.lblAddress.Text = "Address";
			// 
			// listBoxCustomers
			// 
			this.listBoxCustomers.HorizontalScrollbar = true;
			this.listBoxCustomers.Location = new System.Drawing.Point(24, 31);
			this.listBoxCustomers.Name = "listBoxCustomers";
			this.listBoxCustomers.Size = new System.Drawing.Size(463, 108);
			this.listBoxCustomers.Sorted = true;
			this.listBoxCustomers.TabIndex = 3;
			this.listBoxCustomers.SelectedIndexChanged += new System.EventHandler(this.listBoxCustomers_SelectedIndexChanged);
			// 
			// txtPhone
			// 
			this.txtPhone.Location = new System.Drawing.Point(326, 324);
			this.txtPhone.Name = "txtPhone";
			this.txtPhone.Size = new System.Drawing.Size(160, 20);
			this.txtPhone.TabIndex = 18;
			this.txtPhone.Text = "";
			// 
			// btnNew
			// 
			this.btnNew.Location = new System.Drawing.Point(503, 284);
			this.btnNew.Name = "btnNew";
			this.btnNew.Size = new System.Drawing.Size(54, 23);
			this.btnNew.TabIndex = 25;
			this.btnNew.Text = "New";
			this.btnNew.Click += new System.EventHandler(this.btnNew_Click);
			// 
			// lblCompanyID
			// 
			this.lblCompanyID.Location = new System.Drawing.Point(24, 241);
			this.lblCompanyID.Name = "lblCompanyID";
			this.lblCompanyID.RightToLeft = System.Windows.Forms.RightToLeft.Yes;
			this.lblCompanyID.Size = new System.Drawing.Size(82, 13);
			this.lblCompanyID.TabIndex = 6;
			this.lblCompanyID.Text = "Company ID";
			// 
			// lblCompanyName
			// 
			this.lblCompanyName.Location = new System.Drawing.Point(18, 269);
			this.lblCompanyName.Name = "lblCompanyName";
			this.lblCompanyName.RightToLeft = System.Windows.Forms.RightToLeft.Yes;
			this.lblCompanyName.Size = new System.Drawing.Size(88, 13);
			this.lblCompanyName.TabIndex = 8;
			this.lblCompanyName.Text = "Company Name";
			// 
			// lblContactName
			// 
			this.lblContactName.Location = new System.Drawing.Point(24, 298);
			this.lblContactName.Name = "lblContactName";
			this.lblContactName.RightToLeft = System.Windows.Forms.RightToLeft.Yes;
			this.lblContactName.Size = new System.Drawing.Size(82, 13);
			this.lblContactName.TabIndex = 10;
			this.lblContactName.Text = "Contact Name";
			// 
			// grpBoxUpdateCustomer
			// 
			this.grpBoxUpdateCustomer.Controls.AddRange(new System.Windows.Forms.Control[] {
																							   this.btnDelete});
			this.grpBoxUpdateCustomer.ImeMode = System.Windows.Forms.ImeMode.On;
			this.grpBoxUpdateCustomer.Location = new System.Drawing.Point(10, 7);
			this.grpBoxUpdateCustomer.Name = "grpBoxUpdateCustomer";
			this.grpBoxUpdateCustomer.Size = new System.Drawing.Size(567, 194);
			this.grpBoxUpdateCustomer.TabIndex = 29;
			this.grpBoxUpdateCustomer.TabStop = false;
			this.grpBoxUpdateCustomer.Text = "Update selected Customer Name or delete selected Customer";
			// 
			// grpBoxNewCustomer
			// 
			this.grpBoxNewCustomer.Location = new System.Drawing.Point(10, 218);
			this.grpBoxNewCustomer.Name = "grpBoxNewCustomer";
			this.grpBoxNewCustomer.Size = new System.Drawing.Size(566, 145);
			this.grpBoxNewCustomer.TabIndex = 30;
			this.grpBoxNewCustomer.TabStop = false;
			this.grpBoxNewCustomer.Text = "New Customer";
			// 
			// lblMessage
			// 
			this.lblMessage.Location = new System.Drawing.Point(13, 374);
			this.lblMessage.Name = "lblMessage";
			this.lblMessage.Size = new System.Drawing.Size(556, 13);
			this.lblMessage.TabIndex = 31;
			// 
			// CustTrans
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(584, 396);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.txtCustomerName,
																		  this.lblNewCustomerName,
																		  this.lblMessage,
																		  this.txtPhone,
																		  this.lblPhone,
																		  this.txtContactTitle,
																		  this.lblContactTitle,
																		  this.txtZip,
																		  this.lblZip,
																		  this.txtCity,
																		  this.lblCity,
																		  this.txtAddress,
																		  this.lblAddress,
																		  this.txtContactName,
																		  this.lblContactName,
																		  this.txtCompanyName,
																		  this.lblCompanyName,
																		  this.txtCompanyID,
																		  this.lblCompanyID,
																		  this.btnNew,
																		  this.btnUpdate,
																		  this.listBoxCustomers,
																		  this.grpBoxUpdateCustomer,
																		  this.grpBoxNewCustomer});
			this.Name = "CustTrans";
			this.Text = "Update Data using Windows Form and  .NET Data Sets";
			this.grpBoxUpdateCustomer.ResumeLayout(false);
			this.ResumeLayout(false);

		}

		// **** Handle the NEW button click
		protected void btnNew_Click (object sender, System.EventArgs e)
		{
			// Create a new row, populate it with entered
			// Data from the ListBox
			DataRow newRow = _dataTable.NewRow();
			newRow["CustomerID"]   = txtCompanyID.Text;
			newRow["CompanyName"]  = txtCompanyName.Text;
			newRow["ContactName"]  = txtContactName.Text;
			newRow["ContactTitle"] = txtContactTitle.Text;
			newRow["Address"]      = txtAddress.Text;
			newRow["City"]         = txtCity.Text;
			newRow["PostalCode"]   = txtZip.Text;
			newRow["Phone"]        = txtPhone.Text;

			// Add the new row to the table in the dataset
			_dataTable.Rows.Add(newRow);

			// Update the database, any Error is catched in catch block
			try
			{
				// Prepared UPDATE Command is executed
				_dataAdapter.Update(_dataSet,"Customers");
				_dataSet.AcceptChanges();

				// Inform the user and repaint Form
				lblMessage.Text = "Updated!";
				Application.DoEvents();

				// Repopulate the list box
				PopulateListBox();

				// Clear all the text fields
				ClearFields();
			}
			catch (SqlException ex)
			{
				_dataSet.RejectChanges();
				MessageBox.Show(ex.Message);
			}
		}

		// **** Handle the UPDATE button click
		protected void btnUpdate_Click (object sender, System.EventArgs e)
		{
			int  index;
			
			// Get the selected row from the ListBox
			index = listBoxCustomers.SelectedIndex;
			if (index == (-1)) 
			{
				lblMessage.Text = "Please select a Customer";
				return;
			}
			else 
			{
				lblMessage.Text = "";
			}
			DataRow targetRow  = _dataTable.Rows[index];

			// Inform the user
			lblMessage.Text = "Updating: " +  targetRow["CompanyName"];
			Application.DoEvents();

			// Edit the row, no validation is fired between
			// BeginEdit() and EndEdit(). The underlying Dataset
			// will no mark the Row as an UPDATE!
			targetRow.BeginEdit();
			targetRow["CompanyName"] = txtCustomerName.Text;
			targetRow.EndEdit();

			// Use the GetChanges method to create a second DataSet
			// object that is then used to update a data source.
			DataSet dataSetChanged =
				_dataSet.GetChanges(DataRowState.Modified);

			// Test to make sure all the changed rows are without errors
			bool okayFlag = true;
			if (dataSetChanged.HasErrors)
			{
				okayFlag = false;
				string msg = "Error in row with customer ID ";

				// Examine each table in the changed DataSet
				foreach (DataTable theTable in dataSetChanged.Tables)
				{
					// If any table has errors, find out which rows
					if (theTable.HasErrors)
					{
						// Get the rows with errors
						DataRow[] errorRows = theTable.GetErrors();

						// iterate through the errors and correct
						// (in our case, just identify)
						foreach (DataRow theRow in errorRows)
						{
							msg = msg + theRow["CustomerID"];
						}
					}
				}
				lblMessage.Text = msg;
			}
			// If we have no errors
			if (okayFlag)
			{
				// Update the database on second dataset
				_dataAdapter.Update(dataSetChanged,"Customers");

				// Inform the user
				lblMessage.Text = "Updated " +  targetRow["CompanyName"];
				Application.DoEvents();

				// Accept the changes and repopulate the list box
				_dataSet.AcceptChanges();
				PopulateListBox();
			}				
			else  
			{	// If we had errors, reject the changes
				_dataSet.RejectChanges();
			}
		}

		// **** Handle the DELETE button click
		protected void btnDelete_Click (object sender, System.EventArgs e)
		{
			// Get the selected row
			DataRow targetRow  = _dataTable.Rows[listBoxCustomers.SelectedIndex];

			// Prepare message for user
			string msg = targetRow["CompanyName"] + " deleted. ";

			// Delete the selected row
			targetRow.Delete();

			// Update the database
			try
			{
				_dataAdapter.Update(_dataSet,"Customers");
				_dataSet.AcceptChanges();

				// Repopulate the list box without the deleted record
				PopulateListBox();

				// Inform the user
				lblMessage.Text = msg;
				Application.DoEvents();
			}
			catch (SqlException ex)
			{
				_dataSet.RejectChanges();
				MessageBox.Show(ex.Message);
			}

		}

		// **** Get the selected row and show Customer Name in TextBox
		private void listBoxCustomers_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			DataRow targetRow  = _dataTable.Rows[listBoxCustomers.SelectedIndex];
			txtCustomerName.Text = targetRow["CompanyName"].ToString();
		}

		// **** Cleanup when exiting
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		// **** Entry Point
		public static void Main(string[] args)
		{
			Application.Run(new CustTrans());
		}
	}
}